package algorithm

import (
	"fmt"
	"testing"
)

type people struct {
	Name string
	Age  int
}

type peopleAgeComparator struct{}

func (pc *peopleAgeComparator) Compare(v1 any, v2 any) int {
	p1, _ := v1.(people)
	p2, _ := v2.(people)

	// 升序
	if p1.Age < p2.Age {
		return -1
	} else if p1.Age > p2.Age {
		return 1
	}
	return 0
}

type intComparator struct{}

func (c *intComparator) Compare(v1 any, v2 any) int {
	val1, _ := v1.(int)
	val2, _ := v2.(int)

	//ascending order
	if val1 < val2 {
		return -1
	} else if val1 > val2 {
		return 1
	}
	return 0
}

func TestBubbleSortForStructSlice(t *testing.T) {
	t.Parallel()

	peoples := []people{
		{Name: "a", Age: 20},
		{Name: "b", Age: 10},
		{Name: "c", Age: 17},
		{Name: "d", Age: 8},
		{Name: "e", Age: 28},
	}
	comparator := &peopleAgeComparator{}
	BubbleSort(peoples, comparator)

	expected := "[{d 8} {b 10} {c 17} {a 20} {e 28}]"
	actual := fmt.Sprintf("%v", peoples)

	if expected != actual {
		t.Errorf("TestInsertionSort failed, expected %v, got %v", expected, actual)
	}

}

func TestBubbleSortForIntSlice(t *testing.T) {
	t.Parallel()

	numbers := []int{2, 1, 5, 3, 6, 4}
	comparator := &intComparator{}
	BubbleSort(numbers, comparator)
}

func TestInsertionSort(t *testing.T) {
	t.Parallel()

	peoples := []people{
		{Name: "a", Age: 20},
		{Name: "b", Age: 10},
		{Name: "c", Age: 17},
		{Name: "d", Age: 8},
		{Name: "e", Age: 28},
	}
	comparator := &peopleAgeComparator{}
	InsertionSort(peoples, comparator)

	expected := "[{d 8} {b 10} {c 17} {a 20} {e 28}]"
	actual := fmt.Sprintf("%v", peoples)

	if expected != actual {
		t.Errorf("TestInsertionSort failed, expected %v, got %v", expected, actual)
	}
}

func TestSelectionSort(t *testing.T) {
	t.Parallel()

	peoples := []people{
		{Name: "a", Age: 20},
		{Name: "b", Age: 10},
		{Name: "c", Age: 17},
		{Name: "d", Age: 8},
		{Name: "e", Age: 28},
	}
	comparator := &peopleAgeComparator{}
	SelectionSort(peoples, comparator)

	expected := "[{d 8} {b 10} {c 17} {a 20} {e 28}]"
	actual := fmt.Sprintf("%v", peoples)

	if expected != actual {
		t.Errorf("TestSelectionSort failed, expected %v, got %v", expected, actual)
	}
}

func TestShellSort(t *testing.T) {
	t.Parallel()

	peoples := []people{
		{Name: "a", Age: 20},
		{Name: "b", Age: 10},
		{Name: "c", Age: 17},
		{Name: "d", Age: 8},
		{Name: "e", Age: 28},
	}
	comparator := &peopleAgeComparator{}
	ShellSort(peoples, comparator)

	expected := "[{d 8} {b 10} {c 17} {a 20} {e 28}]"
	actual := fmt.Sprintf("%v", peoples)

	if expected != actual {
		t.Errorf("TestSelectionSort failed, expected %v, got %v", expected, actual)
	}
}

func TestQuickSort(t *testing.T) {
	t.Parallel()

	peoples := []people{
		{Name: "a", Age: 20},
		{Name: "b", Age: 10},
		{Name: "c", Age: 17},
		{Name: "d", Age: 8},
		{Name: "e", Age: 28},
	}
	comparator := &peopleAgeComparator{}
	QuickSort(peoples, comparator)

	expected := "[{d 8} {b 10} {c 17} {a 20} {e 28}]"
	actual := fmt.Sprintf("%v", peoples)

	if expected != actual {
		t.Errorf("TestSelectionSort failed, expected %v, got %v", expected, actual)
	}
}

func TestHeapSort(t *testing.T) {
	t.Parallel()

	peoples := []people{
		{Name: "a", Age: 20},
		{Name: "b", Age: 10},
		{Name: "c", Age: 17},
		{Name: "d", Age: 8},
		{Name: "e", Age: 28},
	}
	comparator := &peopleAgeComparator{}
	HeapSort(peoples, comparator)

	expected := "[{d 8} {b 10} {c 17} {a 20} {e 28}]"
	actual := fmt.Sprintf("%v", peoples)

	if expected != actual {
		t.Errorf("TestSelectionSort failed, expected %v, got %v", expected, actual)
	}
}

func TestMergeSort(t *testing.T) {
	t.Parallel()

	peoples := []people{
		{Name: "a", Age: 20},
		{Name: "b", Age: 10},
		{Name: "c", Age: 17},
		{Name: "d", Age: 8},
		{Name: "e", Age: 28},
	}
	comparator := &peopleAgeComparator{}
	MergeSort(peoples, comparator)

	expected := "[{d 8} {b 10} {c 17} {a 20} {e 28}]"
	actual := fmt.Sprintf("%v", peoples)

	if expected != actual {
		t.Errorf("TestSelectionSort failed, expected %v, got %v", expected, actual)
	}
}

func TestCountSort(t *testing.T) {
	t.Parallel()
	peoples := []people{
		{Name: "a", Age: 20},
		{Name: "b", Age: 10},
		{Name: "c", Age: 17},
		{Name: "d", Age: 8},
		{Name: "e", Age: 28},
	}
	comparator := &peopleAgeComparator{}
	sortedPeopleByAge := CountSort(peoples, comparator)

	expected := "[{d 8} {b 10} {c 17} {a 20} {e 28}]"
	actual := fmt.Sprintf("%v", sortedPeopleByAge)

	if expected != actual {
		t.Errorf("TestSelectionSort failed, expected %v, got %v", expected, actual)
	}
}
