package middleware

import (
	"context"
	"git.downtown8.com/golang/common/onelog"
	"github.com/cloudwego/kitex/pkg/endpoint"
	"github.com/cloudwego/kitex/pkg/remote/trans/nphttp2/metadata"
	"github.com/cloudwego/kitex/pkg/streaming"
)

type wrappedStream struct {
	streaming.Stream
}

func (w *wrappedStream) RecvMsg(m interface{}) error {
	traceID := ""
	if md, ok := metadata.FromIncomingContext(w.Context()); ok {
		if mTraceId, ok := md["trace_id"]; ok && len(mTraceId) > 0 {
			traceID = mTraceId[0]
		}
	}
	err := w.Stream.RecvMsg(m)
	onelog.CtxInfoOption(w.Context(), m, "trace_id", traceID)
	return err
}

func (w *wrappedStream) SendMsg(m interface{}) error {
	traceID := ""
	if md, ok := metadata.FromIncomingContext(w.Context()); ok {
		if mTraceId, ok := md["trace_id"]; ok && len(mTraceId) > 0 {
			traceID = mTraceId[0]
		}
	}
	onelog.CtxInfoOption(w.Context(), m, "trace_id", traceID)
	return w.Stream.SendMsg(m)
}

func newWrappedStream(s streaming.Stream) streaming.Stream {
	return &wrappedStream{s}
}

// LoggingMiddleware is a middleware that logs the complete gRPC request
func LoggingMiddleware(next endpoint.Endpoint) endpoint.Endpoint {
	return func(ctx context.Context, req, res interface{}) error {
		//rpcInfo := rpcinfo.GetRPCInfo(ctx)
		//methodName := rpcInfo.Invocation().MethodName()
		var Nil interface{} // go 里不能直接 switch nil
		switch Nil {
		case req: // 当前中间件用于客户端，并且是 streaming 方法
			err := next(ctx, req, res)
			// stream 对象要在最终 endpoint return 后才能获取
			if tmp, ok := res.(*streaming.Result); err == nil && ok {
				//tmp.Stream.SetHeader(metadata.Pairs("traceID", traceID))
				tmp.Stream = newWrappedStream(tmp.Stream) // 包装 stream 对象
			}
			return err
		case res: // 当前中间件用于服务端，并且是 streaming 方法
			if tmp, ok := req.(*streaming.Args); ok {
				//tmp.Stream.SetHeader(metadata.Pairs("traceID", traceID))
				tmp.Stream = newWrappedStream(tmp.Stream) // 包装 stream 对象
			}
		default: // 纯 unary 方法，或 thrift 方法
			// do something else
		}
		//onelog.Info(methodName, "error")
		return next(ctx, req, res)
	}
}
