package one_orm

// Device 结构体定义
type Order struct {
	OrderNo         string           `json:"orderNo"`         // 订单号
	Nono            string           `json:"nono"`            // nono号
	OutOrderNo      string           `json:"outOrderNo"`      // 外部订单号
	OutOrderNono    string           `json:"outOrderNono"`    // 外部nono号
	Bo              string           `json:"bo"`              // 区域：CHN中国；US美国
	BrandId         int64            `json:"brandId"`         // 品牌ID
	StoreId         int64            `json:"storeId"`         // 门店ID
	DeviceNo        string           `json:"deviceNo"`        // 下单设备ID
	Client          string           `json:"client"`          // 下单客户端：pos；mobile手机；elm饿了么；meituan美团；dy抖音
	Scene           string           `json:"scene"`           // 下单场景：dinein堂食；selfPickup自提；3rdDelivery三方外卖；selfDelivery自营外卖
	PayModel        string           `json:"payModel"`        // 付费方式：front先付费；after后付费
	UseType         string           `json:"useType"`         // 场景类型。dc点餐；cz充值
	IsBuyingGroup   bool             `json:"isBuyingGroup"`   // 是否拼单
	CustomerInfo    *CustomerInfo    `json:"customerInfo"`    // 客户信息
	MemberId        int32            `json:"memberId"`        // 会员ID
	MemberCardId    int32            `json:"memberCardId"`    // 会员卡ID
	DineNum         int32            `json:"dineNum"`         // 用餐人数（大人）
	DineChildNum    int32            `json:"dineChildNum"`    // 用餐人数（小孩）
	TableId         int32            `json:"tableId"`         // 桌台ID
	TableNo         string           `json:"tableNo"`         // 桌台号
	MealNum         int32            `json:"mealNum"`         // 牌号
	ReserveTime     string           `json:"reserveTime"`     // 预约时间
	BizRemark       string           `json:"bizRemark"`       // 订单备注
	MakeMethod      string           `json:"makeMethod"`      // 订单出品方式
	Status          string           `json:"status"`          // 订单当前状态
	OriginAmount    float64          `json:"originAmount"`    // 原金额
	DiscountAmount  float64          `json:"discountAmount"`  // 折扣金额
	RealAmount      float64          `json:"realAmount"`      // 实付金额
	IncomeAmount    float64          `json:"incomeAmount"`    // 实收金额（外部订单抽佣）
	PackPk          string           `json:"packPk"`          // 前端打包盒业务ID
	OrderPackAmount float64          `json:"orderPackAmount"` // 订单打包费金额（订单维度）
	PackAmount      float64          `json:"packAmount"`      // 打包费总金额（订单维度+订单餐品维度）
	PayTime         string           `json:"payTime"`         // 支付时间
	CancelTime      string           `json:"cancelTime"`      // 取消时间
	PaymentList     []*PaymentDetail `json:"paymentList"`     // 支付明细列表
	DiscountDetail  *DiscountDetail  `json:"discountDetail"`  // 折扣明细

	//*SystemField
}

type CustomerInfo struct {
	Name    string `json:"name"`    // 名称
	Mobile  string `json:"mobile"`  // 电话
	Address string `json:"address"` // 地址
}

type PaymentDetail struct {
	PayNo     string  `json:"payNo"`     // 支付单号ID
	PayTime   string  `json:"payTime"`   // 支付时间
	PayWay    string  `json:"payWay"`    // 支付方式
	PayAmount float64 `json:"payAmount"` // 支付金额
}

type DiscountDetail struct {
	DiscountNo       string  `json:"discountNo"`       // 折扣ID
	Type             string  `json:"type"`             // 类型
	Method           string  `json:"method"`           // 折扣方式：discount打折；reduce减价
	Amount           float64 `json:"amount"`           // 金额
	AdditionalEnable bool    `json:"additionalEnable"` // 是否包含加料
	Reason           string  `json:"reason"`           // 打折原因
}

type OrderItem struct {
	OrderItemNo     string                   `json:"orderItemNo"`     // 订单明细ID
	OrderNo         string                   `json:"orderNo"`         // 订单ID
	ItemPk          string                   `json:"itemPk"`          // 前端餐品单项业务ID
	SuType          string                   `json:"suType"`          // 类型：comboSu套餐；singleSu餐品；
	SuNo            string                   `json:"suNo"`            //
	ParentItemNo    string                   `json:"parentItemNo"`    // 上级订单明细ID（默认为空表示最高层级）
	SkuNo           string                   `json:"skuNo"`           // skuID（仅suType=singleSu）
	Name            string                   `json:"name"`            // 名称
	SkuName         string                   `json:"skuName"`         // sku名称（仅suType=singleSu）
	Count           int32                    `json:"count"`           // 下单数量
	OutSpuName      string                   `json:"outSpuName"`      // 三方SPU名称
	OutSkuName      string                   `json:"outSkuName"`      // 三方SKU名称
	PackNo          string                   `json:"packNo"`          // 打包费No（仅suType=singleSu）
	PackName        string                   `json:"packName"`        // 打包费名称
	PackPrice       float64                  `json:"packPrice"`       // 打包费金额（仅suType=singleSu）
	ReserveTime     string                   `json:"reserveTime"`     // 预约时间（仅parentItemNo=""）
	AddPrice        float64                  `json:"addPrice"`        // 加价金额
	OriginPrice     float64                  `json:"originPrice"`     // 原单价（仅parentItemNo=""）
	OriginAmount    float64                  `json:"originAmount"`    // 原小计金额（仅parentItemNo=""）
	DiscountAmount  float64                  `json:"discountAmount"`  // 折扣金额
	RealAmount      float64                  `json:"realAmount"`      // 实收金额（仅parentItemNo=""）
	AdditionalItems []*AdditionalItem        `json:"additionalItems"` // 加料明细列表（仅suType=singleSu）
	DiscountDetail  *OrderItemDiscountDetail `json:"discountDetail"`  // 折扣明细

}

type AdditionalItem struct {
	ItemNo         string  `json:"itemNo"`         // 加料项ID
	Name           string  `json:"name"`           // 加料项名称
	SkuNo          string  `json:"skuNo"`          //
	SpuNo          string  `json:"spuNo"`          //
	Price          float64 `json:"price"`          // 加料价格
	Count          int32   `json:"count"`          // 下单加料数量
	DiscountAmount float64 `json:"discountAmount"` // 折扣金额
}

type OrderItemDiscountDetail struct {
	DiscountNo       string  `json:"discountNo"`       // 折扣ID
	Type             string  `json:"type"`             // 类型
	Method           string  `json:"method"`           // 折扣方式：discount打折；reduce减价
	Amount           float64 `json:"amount"`           // 金额
	AdditionalEnable bool    `json:"additionalEnable"` // 是否包含加料
	Reason           string  `json:"reason"`           // 打折原因
}

type Voice struct {
	IsEnable  bool   `json:"isEnable"`
	VoiceId   string `json:"voiceId"`
	VoiceType string `json:"voiceType"` //  1.叮叮 2.详情播报 3.欢快鼓播报 4.古风 5.可爱卡通
	VoiceUrl  string `json:"voiceUrl"`
	PlayNum   int32  `json:"playNum"` // 播放次数
}

type Device struct {
	StoreDeviceId  string `json:"storeDeviceId"`
	StoreId        int32  `json:"storeId"`
	BrandId        int32  `json:"brandId"`
	Name           string `json:"name"`
	DeviceType     string `json:"deviceType"` // 设备类型  pos 、kds 、 expo
	DeviceNo       string `json:"deviceNo"`   // 设备code
	DeviceId       string `json:"deviceId"`
	Status         string `json:"status"` // 设备状态 normal 和 unbind
	Ip             string `json:"ip"`
	System         string `json:"system"`
	SystemVersion  string `json:"systemVersion"`
	SystemModel    string `json:"systemModel"`
	Provider       string `json:"provider"`
	AppVersion     string `json:"appVersion"`
	AppCodeVersion string `json:"appCodeVersion"`
	IsAuthPrint    bool   `json:"isAuthPrint"`
	PrintRule      int32  `json:"printRule"` // 设备打印规则 0 设备打印纸规格 1 虚拟打印机状态
	PrinterId      string `json:"printerId"`
	AddTime        string `json:"addTime"`
	IsDel          bool   `json:"isDel"`
	ModifyTime     string `json:"modifyTime"`
	//DeletedTime    time.Time `json:"deletedTime"`
	ExpiryTime string `json:"expiryTime"`
	TraceId    string `json:"traceId"`

	PosOrder       Voice `json:"posOrder"`
	MeiTuanTakeout Voice `json:"meiTuanTakeout"`
	EleMe          Voice `json:"eleMe"`
	NetWorkError   Voice `json:"netWorkError"`
	PrintError     Voice `json:"printError"`
	PrinterVoice   bool  `json:"printerVoice"`
}

//func main() {
//	cluster, err := gocb.Connect("192.168.105.13", gocb.ClusterOptions{
//		Username: "dev",
//		Password: "downtown2013",
//	})
//	if err != nil {
//		panic(err)
//	}
//	bucket := cluster.Bucket("base")
//	scope := bucket.Scope("device")
//	collection := scope.Collection("device")
//	//collection1 := scope.Collection("orderItem")
//	db := NewCouchbaseDB(cluster, bucket, scope, collection)
//	query1 := NewQueryBuilder[Order](db)
//	var documents []*Device
//
//	err = query1.Model(&Device{}).
//		Select("device.storeId").
//		Where("storeId = ?", 200).
//		GroupBy("storeId").
//		Find(&documents)
//	for _, doc := range documents {
//		fmt.Println(doc.StoreId)
//		fmt.Println(doc.DeviceId)
//	}
//	fmt.Println(err, 111)
//	fmt.Printf("First device: %+v\n", documents)
//}

//
//func main() {
//	//	// Initialize Couchbase connection
//	cluster, err := gocb.Connect("192.168.105.13", gocb.ClusterOptions{
//		Username: "dev",
//		Password: "downtown2013",
//	})
//	if err != nil {
//		panic(err)
//	}
//
//	bucket := cluster.Bucket("base")
//	scope := bucket.Scope("test")
//	collection := scope.Collection("test")
//	collection1 := scope.Collection("test1")
//	//collection1 := scope.Collection("orderItem")
//	db := NewCouchbaseDB(cluster, bucket, scope, collection)
//	m := map[string]interface{}{}
//	m["0297ca22-5cd1-405f-a0f2-68141da4a1e8"] = &PaymentDetail{
//		PayNo:     "0297ca22-5cd1-405f-a0f2-68141da4a1e8",
//		PayTime:   "2024-08-05T00:00:00",
//		PayWay:    "z11",
//		PayAmount: 103,
//	}
//	tm := NewTransactionManager(db)
//	err = tm.RunInTransaction(func(ctx *gocb.TransactionAttemptContext) error {
//		tm.BatchUpsertInTransaction(ctx, collection, m)
//		tm.BatchUpsertInTransaction(ctx, collection1, m)
//		return nil
//	}, &gocb.TransactionOptions{
//		DurabilityLevel: gocb.DurabilityLevelMajority,
//	})
//	//uid1 := uuid2.New().String()
//	//m["9d5e954d-c4e3-465c-b59f-142d4500cbf6"] = &PaymentDetail{
//	//	PayNo:     "9d5e954d-c4e3-465c-b59f-142d4500cbf6",
//	//	PayTime:   "2024-08-06T00:00:00",
//	//	PayWay:    "a8",
//	//	PayAmount: 101,
//	//}
//	////uid2 := uuid2.New().String()
//	//m["b62684ff-08fa-4d82-8c02-bce620312efa"] = &PaymentDetail{
//	//	PayNo:     "b62684ff-08fa-4d82-8c02-bce620312efa",
//	//	PayTime:   "2024-08-05700:00:00",
//	//	PayWay:    "c9",
//	//	PayAmount: 103,
//	//}
//
//	//	//db1 := NewCouchbaseDB(cluster, bucket, scope, collection1)
//	//
//	//	//// 示例：获取第一条记录
//	//	//var firstDevice Device
//	//	//db.Model(&Device{}).
//	//	//	Table("device").
//	//	//Select("storeDeviceId", "storeId", "brandId", "name").
//	//	//Where("storeId > $param1", 1).
//	//	//Order("addTime DESC").
//	//	//First(&firstDevice)
//	//	//fmt.Printf("First device: %+v\n", firstDevice)
//	//	//
//	//	//// 示例：获取记录总数
//	//	//var orders []*OrderItem
//	//	//var count int
//	//	//err = db.NewQueryBuilder().Model(&Device{}).
//	//	//Table("device").
//	//	//Select("name").
//	//	//Where("brandId = $param1", 300).
//	//	//Count(&count, "counter"). // todo 如果带field参数的count 必须在where之后
//	//	//Order("addTime DESC").
//	//	//Limit(10).Find(&devices).Exec()
//	//	//brandId := 7000001
//	//	//storeId := 800738
//	//	//deviceId := "12312"
//	//	//Query:  SELECT orderItem.* FROM `order`.`order`.`orderItem` WHERE orderNo = ?
//	//	//	Query Parameters:  [2407250648418022453164934524]
//	//	//err = NewQueryBuilder[OrderItem](db).Select("`orderItem`.*").
//	//	//	Model(&OrderItem{}).
//	//	//	Where("orderNo = ?", "2407250648418022453164934524").
//	//	//	//ScanConsistency(gocb.QueryScanConsistencyRequestPlus).
//	//	//	//Where("storeId = ?", 9000041).
//	//	//	//Where("isDel = ?", false).
//	//	//	//Where("addTime >= ?", "2024-08-05T00:00:00").
//	//	//	//Where("addTime <= ?", "2024-08-05T23:59:59").
//	//	//	//Count(&count, "").
//	//	//	Find(&orders)
//	//	//if err != nil {
//	//	//	fmt.Println("Error:", err)
//	//	//} else {
//	//	//	//for _, d := range devices {
//	//	//	//	fmt.Printf("device: %+v\n", d)
//	//	//	//}
//	//	//}
//	//	//fmt.Println(orders)
//	//	////for _, d := range devices {
//	//	////	fmt.Printf("device: %+v\n", d)
//	//	////}
//	//	//fmt.Println(count)
//	//	//{
//	//	//
//	//	//Where("brandId = $param1", 7000001).
//	//	//Where("storeId = $param2", 800738).
//	//	//Where("isDel = true").
//	//	//Count(&count, "counter").
//	//	//Find(&devices)
//	//	//if brandId > 0 {
//	//	//	query = query.Where("brandId = ?", brandId)
//	//	//}
//	//	//if deviceId != "" {
//	//	//	query = query.Where("deviceId = $param2", deviceId)
//	//	//}
//	//	//if storeId > 0 {
//	//	//	query = query.Where("storeId = ?", storeId)
//	//	//}
//	//
//	//	//err = query.Where("isDel = ? and isAuthPrint = ?", true, true).Count(&count, "counter").Find(&devices)
//	//	//fmt.Println(count)
//	//	//fmt.Println(devices)
//	//	//fmt.Println(err)
//	//	//if err != nil {
//	//	//	return
//	//	//}
//	//
//	//	//}
//	//
//	//	//query1 := NewQueryBuilder[Order](db)
//	//	//err = query1.RawQuery("SELECT `order`.*  FROM `rms`.`order`.`order` WHERE brandId = 7000124 AND storeId = 9000033 AND isDel = false AND status in [\"paid\"] AND addTime >= \"2024-05-12T00:00:00\" AND addTime <= \"2024-08-12T23:59:59\"").Find(&orders)
//	//
//	//	//sql := "SELECT `order`.*  FROM `rms`.`order`.`order` WHERE brandId = $param1 AND storeId = $param2 AND isDel = $param3  AND addTime >= $param4 AND addTime <= $param5"
//	//	//rows, err := cluster.Query(sql, &gocb.QueryOptions{
//	//	//	NamedParameters: map[string]interface{}{
//	//	//		"$param1": 7000124,
//	//	//		"$param2": 9000033,
//	//	//		"$param3": false,
//	//	//		"$param4": "2024-05-12T00:00:00",
//	//	//		"$param5": "2024-08-12T23:59:59",
//	//	//	},
//	//	//})
//	//	//rows, err := cluster.Query(sql, nil)
//	//	//defer rows.Close()
//	//
//	//	//destVal := reflect.ValueOf(orders)
//	//	//if destVal.Kind() != reflect.Ptr || destVal.Elem().Kind() != reflect.Slice {
//	//	//	return
//	//	//}
//	//	//sliceVal := destVal.Elem()
//	//	//elemType := sliceVal.Type().Elem()
//	//
//	//	//for rows.Next() {
//	//	//	var document Order
//	//	//	err := rows.Row(&document)
//	//	//	if err != nil {
//	//	//		return
//	//	//	}
//	//	//	orders = append(orders, &document)
//	//	//}
//	//	//if err := rows.Err(); err != nil {
//	//	//	return
//	//	//}
//	//	//fmt.Println(orders)
//	//	//
//	//	//if err != nil {
//	//	//	fmt.Println("Error:", err)
//	//	//}
//	//	//for _, d := range orders {
//	//	//	fmt.Printf("device: %+v\n", d)
//	//	//}
//	//	//fmt.Println(orders)
//	//	//fmt.Println(count)
//	//	//for _, d := range devices {
//	//	//	fmt.Printf("device: %+v\n", d)
//	//	//}
//	//	//fmt.Println(devices, 11111)
//	//
//	//	//打印总耗时
//	//	//
//	//	// 示例查询
//	//	var devices []Device
//	//	err = db.Model(&Device{}).
//	//		Table("device").
//	//		Select("device.storeId as storeId").
//	//		Join("INNER", "`brand`.`store`.`store_business`", "device.storeId = store_business.storeId").
//	//		Order("device.addTime DESC").
//	//		Find(&devices)
//	//	if err != nil {
//	//		fmt.Println("Error:", err)
//	//	} else {
//	//for _, d := range devices {
//	//	fmt.Printf("device: %+v\n", d)
//	//}
//	//}
//	//	//fmt.Println(devices)
//	//var orders []*Order
//	//var count int
//	//offset := (1 - 1) * 10
//	//startDate := "2024-05-12T00:00:00"
//	//endDate := "2024-08-12T23:59:59"
//	//err = NewQueryBuilder[Order](db).Model(&Order{}).
//	//	Select("`order`.*").
//	//	Where("brandId = ?", 7000124).
//	//	Where("storeId = ?", 9000033).
//	//	Where("isDel = ?", false).
//	//	Where("addTime >= ?", startDate).
//	//	Where("addTime <= ?", endDate).
//	//	Count(&count, "").
//	//	Order("addTime desc").
//	//	Limit(int(10)).
//	//	Offset(int(offset)).
//	//	Count(&count, "").
//	//	Find(&orders)
//	//fmt.Println(err)
//
//	//	var orderItem []*OrderItem
//	//	var count1 int
//	//	err= NewQueryBuilder[OrderItem](db1).Select("orderNo").
//	//		Model(&OrderItem{}).
//	//		Where("orderNo = ?", "2407250648418022453164934524").
//	//		Find(&orderItem)
//	//	//
//	//	startDate := "2024-08-05T00:00:00"
//	//	endDate := "2024-08-05T23:59:59"
//	//
//	//	query = query.Where("addTime >= ? and addTime <= ?", startDate, endDate)
//	//
//	//	err = query.Count(&count, "").
//	//		Order("addTime desc").
//	//		Limit(int(10)).
//	//		Offset(int(offset)).
//	//		Find(&orders)
//	//	if err != nil {
//	//		fmt.Println("Error:", err)
//	//	}
//	//	fmt.Println("orders", orders)
//	//	fmt.Println(orderItem)
//	//	for _, d := range orders {
//	//		fmt.Printf("device: %+v\n", d)
//	//	uid := uuid2.New().String()
//	//	fmt.Println(err)
//}
