package token

import (
	"bytes"
	"fmt"
	"github.com/bytedance/sonic"
	"io/ioutil"
	"net/http"
)

type TokenRequest struct {
	UserId         string  `json:"userId"`
	BrandId        int32   `json:"brandId"`
	StoreId        int32   `json:"storeId"`
	Exp            int64   `json:"exp"`
	TokenID        string  `json:"tokenId"`
	Token          string  `json:"token"`
	BrandIds       []int32 `json:"brandIds"`
	EmployeeRoleId int32   `json:"employeeRoleId"`
	DeviceNo       string  `json:"deviceNo"`
	DeviceId       string  `json:"deviceId"`
	EmployeeId     string  `json:"employeeId"`
}

type TokenResponse struct {
	Token   string `json:"token"`
	Exp     int64  `json:"exp"`
	TokenID string `json:"token_id"`
}

func GenerateToken(urlStr string, request TokenRequest) (TokenResponse, error) {
	var tokenResponse TokenResponse

	// 将请求数据编码成 JSON
	jsonData, err := sonic.Marshal(request)
	if err != nil {
		return tokenResponse, fmt.Errorf("failed to marshal request data: %w", err)
	}

	// 创建一个新的请求
	req, err := http.NewRequest("POST", urlStr, bytes.NewBuffer(jsonData))
	if err != nil {
		return tokenResponse, fmt.Errorf("failed to create request: %w", err)
	}

	// 设置请求头
	req.Header.Set("Content-Type", "application/json")

	client := &http.Client{}

	// 使用 http.Client 发送请求
	resp, err := client.Do(req)
	if err != nil {
		return tokenResponse, fmt.Errorf("failed to send request: %w", err)
	}
	defer resp.Body.Close()

	// 读取响应体
	body, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		return tokenResponse, fmt.Errorf("failed to read response body: %w", err)
	}

	// 检查响应状态码
	if resp.StatusCode != http.StatusOK {
		return tokenResponse, fmt.Errorf("unexpected status code: %d, body: %s", resp.StatusCode, string(body))
	}
	fmt.Println(string(body))
	// 解析响应体
	err = sonic.Unmarshal(body, &tokenResponse)
	if err != nil {
		return tokenResponse, fmt.Errorf("failed to unmarshal response body: %w", err)
	}
	tokenResponse.Exp = request.Exp
	return tokenResponse, nil
}
